/*###################################################################################################
#
#  Author:      Vikram Khatri (vikram@zinox.com)
#
# (C) COPYRIGHT International Business Machines Corp. 2006-2007
#
# All Rights Reserved
#
# This software ("Software") is owned by International Business Machines Corporation or one of its 
# subsidiaries ("IBM") and is copyrighted and licensed, not sold. This Software is not part of any 
# standard IBM product. You may use, copy, modify, and distribute this Software in any form without 
# payment to IBM, for the purpose of assisting you in the development of your applications. This 
# Software is provided to you on an "AS IS" basis, without warranty of any kind. IBM HEREBY EXPRESSLY 
# DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do not allow 
# for the exclusion or limitation of implied warranties, so the above limitations or exclusions may not 
# apply to you. IBM shall not be liable for any damages you suffer as a result of using, copying, 
# modifying or distributing this Software, even if IBM has been advised of the possibility 
# of such damages. 
#
# US Government Users Restricted Rights - Use, duplication or disclosure restricted 
# by GSA ADP Schedule Contract with IBM Corp.
#
###################################################################################################*/
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "splogger.h"

int fenced, notFenced;
       
int printTestSP()
{
	FILE *fp;
	 
    if ((fp = fopen("testsp.sql","w")) == NULL)
    {
    	printf("Error in opening file testsp.sql errno=%d", errno);
        return -1;
    }
	
    fprintf(fp,"--#SET TERMINATOR @\n");
    fprintf(fp,"\n");
    fprintf(fp,"SET CURRENT SCHEMA = 'TEST'\n");
    fprintf(fp,"@\n");
    fprintf(fp,"DROP PROCEDURE TESTSP\n");
    fprintf(fp,"@\n");
    fprintf(fp,"CREATE PROCEDURE TESTSP()\n");
    fprintf(fp,"LANGUAGE SQL\n");
    fprintf(fp,"BEGIN\n");
    fprintf(fp,"   DECLARE h CHAR(%d) FOR BIT DATA;\n", BYTEHANDLESIZE);
    fprintf(fp,"   DECLARE count INTEGER DEFAULT 0;\n");
    fprintf(fp,"   CALL DB2.OPEN_LOG('TESTSP',h);\n");
    fprintf(fp,"   SET count = count + 1;\n");
    fprintf(fp,"   CALL DB2.LOGINFO(h,'this is a test loginfo '||TRIM(CHAR(COALESCE(count,0)))||' message');\n");
    fprintf(fp,"   WHILE (count < 3000)\n");
    fprintf(fp,"     DO\n");
    fprintf(fp,"     SET count = count + 1;\n");
    fprintf(fp,"     CALL DB2.LOGGER(h,'this is a test logger '||TRIM(CHAR(COALESCE(count,0)))||' message');\n");
    fprintf(fp,"   END WHILE;\n");
    fprintf(fp,"   CALL DB2.CLOSE_LOG(h);\n");
    fprintf(fp,"END\n");
    fprintf(fp,"@\n");
    
    fclose(fp);
}

int printSPCAT()
{
	FILE *fp;
	 
    if ((fp = fopen("spcat","w")) == NULL)
    {
    	printf("Error in opening file spcat errno=%d", errno);
    	return -1;
    }
    if (fchmod(fileno(fp), S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP) < 0)
    {
    	printf("Error in setting file persmission for spcat errno%d", errno);
    	return -1;    	
    }
    fprintf(fp,"#! /bin/ksh\n");
    fprintf(fp,"#############################################################################\n");
    fprintf(fp,"# (C) COPYRIGHT International Business Machines Corp. 2008 \n");
    fprintf(fp,"#\n");
    fprintf(fp,"# Vikram Khatri vikram.khatri@us.ibm.com\n");
    fprintf(fp,"#############################################################################\n");
    fprintf(fp,"\n");
    if (fenced)
    {
       fprintf(fp,"export ${FENCED:=\"FENCED NOT THREADSAFE\"}\n");
    }
    if (notFenced)
    {
       fprintf(fp,"export ${FENCED:=\"NOT FENCED THREADSAFE\"}\n");
    }
    fprintf(fp,"export ${SHLIBNAME:=\"splogger\"}\n");
    fprintf(fp,"TOK=$(date +\"%%y%%m%%d%%H%%M%%S\")\n");
    fprintf(fp,"export SHLIBNAME=splogger${TOK}\n");
    if (fenced)
    {
       fprintf(fp,"rm -f ~/sqllib/function/splogger*\n");
       fprintf(fp,"cp -f splogger ~/sqllib/function/${SHLIBNAME}\n");
    }
    if (notFenced)
    {
       fprintf(fp,"rm -f ~/sqllib/function/unfenced/splogger*\n");
       fprintf(fp,"cp -f splogger ~/sqllib/function/unfenced/${SHLIBNAME}\n");
    }
    fprintf(fp,"\n");
    fprintf(fp,"db2 -tv << !EOF\n");
    fprintf(fp,"\n");
    fprintf(fp,"CONNECT TO $1;\n");
    fprintf(fp,"\n");
    fprintf(fp,"DROP PROCEDURE DB2.UPDATE_SP_CONFIG;\n");
    fprintf(fp,"\n");
    fprintf(fp,"CREATE PROCEDURE DB2.UPDATE_SP_CONFIG\n");
    fprintf(fp,"(\n");
    fprintf(fp,"  IN    SP_NAME       VARCHAR(30),\n");
    fprintf(fp,"  IN    MODE          CHAR(1)\n");
    fprintf(fp,")\n");
    fprintf(fp,"SPECIFIC UPDSPCFG_%s\n", (fenced) ? "FENCED" : "TRUSTED");
    fprintf(fp,"DYNAMIC RESULT SETS 0\n");
    fprintf(fp,"DETERMINISTIC\n");
    fprintf(fp,"LANGUAGE C\n");
    fprintf(fp,"PARAMETER STYLE SQL\n");
    fprintf(fp,"DBINFO\n");
    fprintf(fp,"${FENCED}\n");
    fprintf(fp,"NO SQL\n");
    fprintf(fp,"PROGRAM TYPE SUB\n");
    fprintf(fp,"EXTERNAL NAME '${SHLIBNAME}!UpdateSPConfig';\n");
    fprintf(fp,"\n");
    fprintf(fp,"GRANT EXECUTE ON PROCEDURE DB2.UPDATE_SP_CONFIG TO PUBLIC;\n");
    fprintf(fp,"\n");
    fprintf(fp,"DROP PROCEDURE DB2.OPEN_LOG;\n");
    fprintf(fp,"\n");
    fprintf(fp,"CREATE PROCEDURE DB2.OPEN_LOG\n");
    fprintf(fp,"(\n");
    fprintf(fp,"  IN    SP_NAME       VARCHAR(30),\n");
    fprintf(fp,"  OUT   HANDLE        CHAR(%d) FOR BIT DATA \n", BYTEHANDLESIZE);
    fprintf(fp,")\n");
    fprintf(fp,"SPECIFIC OPENLOG_%s\n", (fenced) ? "FENCED" : "TRUSTED");
    fprintf(fp,"DYNAMIC RESULT SETS 0\n");
    fprintf(fp,"DETERMINISTIC\n");
    fprintf(fp,"LANGUAGE C\n");
    fprintf(fp,"PARAMETER STYLE SQL\n");
    fprintf(fp,"DBINFO\n");
    fprintf(fp,"${FENCED}\n");
    fprintf(fp,"NO SQL\n");
    fprintf(fp,"PROGRAM TYPE SUB\n");
    fprintf(fp,"EXTERNAL NAME '${SHLIBNAME}!OpenLog';\n");
    fprintf(fp,"\n");
    fprintf(fp,"GRANT EXECUTE ON PROCEDURE DB2.OPEN_LOG TO PUBLIC;\n");
    fprintf(fp,"\n");
    fprintf(fp,"DROP PROCEDURE DB2.CLOSE_LOG;\n");
    fprintf(fp,"\n");
    fprintf(fp,"CREATE PROCEDURE DB2.CLOSE_LOG\n");
    fprintf(fp,"(\n");
    fprintf(fp,"  INOUT  HANDLE        CHAR(%d) FOR BIT DATA  \n", BYTEHANDLESIZE);
    fprintf(fp,")\n");
    fprintf(fp,"SPECIFIC CLOSELOG_%s\n", (fenced) ? "FENCED" : "TRUSTED");
    fprintf(fp,"DYNAMIC RESULT SETS 0\n");
    fprintf(fp,"DETERMINISTIC\n");
    fprintf(fp,"LANGUAGE C\n");
    fprintf(fp,"PARAMETER STYLE SQL\n");
    fprintf(fp,"DBINFO\n");
    fprintf(fp,"${FENCED}\n");
    fprintf(fp,"NO SQL\n");
    fprintf(fp,"PROGRAM TYPE SUB\n");
    fprintf(fp,"EXTERNAL NAME '${SHLIBNAME}!CloseLog';\n");
    fprintf(fp,"\n");
    fprintf(fp,"GRANT EXECUTE ON PROCEDURE DB2.CLOSE_LOG TO PUBLIC;\n");
    fprintf(fp,"\n");
    fprintf(fp,"DROP PROCEDURE DB2.LOGGER;\n");
    fprintf(fp,"\n");
    fprintf(fp,"CREATE PROCEDURE DB2.LOGGER\n");
    fprintf(fp,"(\n");
    fprintf(fp,"  INOUT    HANDLE       CHAR(%d) FOR BIT DATA,\n", BYTEHANDLESIZE);
    fprintf(fp,"  IN       MSG          VARCHAR(4000)  \n");
    fprintf(fp,")\n");
    fprintf(fp,"SPECIFIC LOGGER_%s\n", (fenced) ? "FENCED" : "TRUSTED");
    fprintf(fp,"DYNAMIC RESULT SETS 0\n");
    fprintf(fp,"DETERMINISTIC\n");
    fprintf(fp,"LANGUAGE C\n");
    fprintf(fp,"PARAMETER STYLE SQL\n");
    fprintf(fp,"DBINFO\n");
    fprintf(fp,"${FENCED}\n");
    fprintf(fp,"NO SQL\n");
    fprintf(fp,"PROGRAM TYPE SUB\n");
    fprintf(fp,"EXTERNAL NAME '${SHLIBNAME}!Logger';\n");
    fprintf(fp,"\n");
    fprintf(fp,"GRANT EXECUTE ON PROCEDURE DB2.LOGGER TO PUBLIC;\n");
    fprintf(fp,"\n");
    fprintf(fp,"DROP PROCEDURE DB2.LOGINFO;\n");
    fprintf(fp,"\n");
    fprintf(fp,"CREATE PROCEDURE DB2.LOGINFO\n");
    fprintf(fp,"(\n");
    fprintf(fp,"  INOUT    HANDLE       CHAR(%d) FOR BIT DATA,\n",BYTEHANDLESIZE);
    fprintf(fp,"  IN       MSG          VARCHAR(4000)  \n");
    fprintf(fp,")\n");
    fprintf(fp,"SPECIFIC LOGINFO_%s\n", (fenced) ? "FENCED" : "TRUSTED");
    fprintf(fp,"DYNAMIC RESULT SETS 0\n");
    fprintf(fp,"DETERMINISTIC\n");
    fprintf(fp,"LANGUAGE C\n");
    fprintf(fp,"PARAMETER STYLE SQL\n");
    fprintf(fp,"DBINFO\n");
    fprintf(fp,"${FENCED}\n");
    fprintf(fp,"NO SQL\n");
    fprintf(fp,"PROGRAM TYPE SUB\n");
    fprintf(fp,"EXTERNAL NAME '${SHLIBNAME}!Loginfo';\n");
    fprintf(fp,"\n");
    fprintf(fp,"GRANT EXECUTE ON PROCEDURE DB2.LOGINFO TO PUBLIC;\n");
    fprintf(fp,"\n");
    fprintf(fp,"!EOF\n");

    fclose(fp);
}

int main(int argc, char *argv[])
{
	char option[30];
	if (argc < 2)
	{
		printf("Usage : %s [FENCED|TRUSTED]\n", argv[0]);
		return -1;
	} else
	{
        fenced = notFenced = 0;
		strcpy(option, argv[1]);
		if (strcmp(option,"FENCED") == 0)
			fenced = 1;
		if (strcmp(option,"TRUSTED") == 0)
			notFenced = 1;
		if (!fenced && !notFenced)
		{
			printf("Invalid option specified.\n");
			printf("Usage : %s [FENCED|TRUSTED]\n", argv[0]);
			return -1;			
		}
	}
	if (printSPCAT() < 0) return -1;
	if (printTestSP() < 0) return -1;
	return 0;
}
