/*###################################################################################################
#
#  Author:      Vikram Khatri (vikram@zinox.com)
#
# (C) COPYRIGHT International Business Machines Corp. 2006-2007
#
# All Rights Reserved
#
# This software ("Software") is owned by International Business Machines Corporation or one of its 
# subsidiaries ("IBM") and is copyrighted and licensed, not sold. This Software is not part of any 
# standard IBM product. You may use, copy, modify, and distribute this Software in any form without 
# payment to IBM, for the purpose of assisting you in the development of your applications. This 
# Software is provided to you on an "AS IS" basis, without warranty of any kind. IBM HEREBY EXPRESSLY 
# DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do not allow 
# for the exclusion or limitation of implied warranties, so the above limitations or exclusions may not 
# apply to you. IBM shall not be liable for any damages you suffer as a result of using, copying, 
# modifying or distributing this Software, even if IBM has been advised of the possibility 
# of such damages. 
#
# US Government Users Restricted Rights - Use, duplication or disclosure restricted 
# by GSA ADP Schedule Contract with IBM Corp.
#
###################################################################################################*/
#define GOFILE
#define DEBUG 0
#include <time.h>
#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/timeb.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/shm.h>
#include <sys/param.h>
#include <pwd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <sqlda.h>
#include <sqlca.h>
#include <sqludf.h>
#include <sql.h>

#include <sqlcodes.h>
#include "splogger.h"

#define SP_SHM_ID 122003
#define SLOG_SHM_SIZE  204800
#define FILE_SIZE 10485760
#define LOG_CONFIG_NAME  ".splogrc"
#define HASHSIZE 1     
#define APPIDSIZE 129
#define LOGFILESIZE 512

#define MAP(c) ((unsigned)c < 128 ? lexmap[(unsigned)c] : 0)

uint lexmap[128];
uint c;
FILE *fin;
char sqlStmtInfo[256];

typedef struct _plist PList;

struct _plist
{
    char name[NAMESIZE+DBSIZE]; /* SP token name + db Name for uniqueness*/
    char debug;  /* Debug Mode either 'Y' or 'N' */
    PList *next; /* linear hash chaining */
};

typedef struct _sphead SPhead;

struct _sphead
{
   char gLogging; /* Global Logging 'Y' On 'N' Off */
   char gLogSplit; /* Log Spilit 'N' for a single file 'Y' for individual files */
   int  gOffset; /* Position where hastable starts */ 
};

typedef struct _logger SLOG;

struct _logger
{
    int shmid;
    char name[NAMESIZE];
    char dbName[DBSIZE];
    char appID[APPIDSIZE];
	char *shmmem;
    Handle handle;
    SPhead *header;
    PList *hashtable[HASHSIZE];
};

unsigned hash(char *s);
int updateSPConfig(SLOG *slog, char *name, char *mode);
int openLog(SLOG *slog, char *name);
int closeLog(SLOG *slog);
int logger(SLOG *slog, char *fmt, ... );
int logginfo(SLOG *slog, char *fmt, ... );
void parseSPConfig(SLOG *slog);
void hardw(char *msg, ...);
char *getTimeTS();

char *strupr(char *t, char *s)
{
   char *p = t, *q = s;
   while (*q)
   {
      *p = toupper(*q);
      ++q;
      ++p;
   }
   *p = '\0';
   return t;
}

char *strlwr(char *t, char *s)
{
   char *p = t, *q = s;
   while (*q)
   {
      *p = tolower(*q);
      ++q;
      ++p;
   }
   *p = '\0';
   return t;
}

double getElapsed(struct timeb *prevTime)
{
	   struct timeb endTime;
	   if (prevTime == NULL)
		   return -1.00;
	   ftime(&endTime);
	   return ((double) endTime.time + ((double) endTime.millitm * 0.001)) -
	     ((double) prevTime->time + ((double) prevTime->millitm * 0.001));	
}

size_t getHomeDir(char *homeDir, size_t bufLen)
{
	char *s;	   
	struct passwd *pwd;
	size_t homeDirLen = 0; 
	   
	*homeDir = '\0';
	pwd = getpwuid(getuid());
	if (pwd != NULL && pwd->pw_dir != NULL &&
			strlen(pwd->pw_dir) < bufLen)
	{
		strcpy(homeDir, pwd->pw_dir);
		homeDirLen = strlen(homeDir);
	} else
	{
		s = getenv("HOME");
		if (s != NULL && strlen(s) < bufLen)
		{
			strcpy(homeDir, pwd->pw_dir);
			homeDirLen = strlen(homeDir);			
		} else
		{
			strcpy(homeDir, "/tmp");
			homeDirLen = strlen(homeDir);						
		}
	}
	/*hardw("getHomeDir %s", homeDir);*/
	return homeDirLen;
}

char *getspName(char *spName)
{
   char *s = strchr(spName,'_');
   return (s == NULL) ? "NULL" : s+1;
}

void printHashTable(SLOG *slog)
{      
      int i;
      PList *next;

      hardw("Start hash table ******");
      for (i = 0; i < HASHSIZE; ++i)
      {
          next = slog->hashtable[i];
  
          while(next)
          {
              hardw("hash=%d %s=%c", hash(next->name), next->name, next->debug);
              next = next->next;
          }
      }
      hardw("End hash table ******");
}

void initSlog(SLOG *slog, SQLUDF_DBINFO *dbInfo)
{
   int i, len;
   char *s;
   
   slog->header = NULL;
   for (i = 0; i < HASHSIZE; ++i)
   {
	   slog->hashtable[i] = NULL;
   }	
   strncpy(slog->dbName, (char *)(dbInfo->dbname), dbInfo->dbnamelen);
   slog->dbName[dbInfo->dbnamelen] = '\0';
   strlwr(slog->dbName, slog->dbName);
   len = strlen(dbInfo->appl_id);
   i = 0;
   if (len > 0)
   {
      s = dbInfo->appl_id + len - 1;
      while (*s)
      {
        if (*s == '.')
          ++i;
        if (i == 2)
           break;
        --s;
      }
      if (s == NULL || *s == '\0')
        strcpy(slog->appID, "NA");
      else
        strcpy(slog->appID,s+1); 
   } else
   {
      if (s == NULL || *s == '\0')
        strcpy(slog->appID, "NA");
   } 
}

void buildHashTable(SLOG *slog)
{
   PList *np;
   int i, size;
   unsigned hashval;

   size = sizeof(SPhead);
   while(size < slog->header->gOffset)
   {
      np = (PList *) (slog->shmmem + size);
      hashval = hash(np->name);
      /*hardw("buildhashtable np->name %s size [%d]%d/%d Flag %c", np->name, hashval, size, slog->header->gOffset, np->debug);*/
      np->next = slog->hashtable[hashval];
      slog->hashtable[hashval] = np;
      size += sizeof(PList);
   }
}

int attachSHM(SLOG *slog)
{
   int listSize, totalSize, headSize;

   /*hardw("In attachSHM");
   hardw("dbName=%s", dbName);*/
   
   listSize = sizeof(PList);
   headSize = sizeof(SPhead);
   
   totalSize = headSize + HASHSIZE * listSize;
   /*hardw("listSize %d headSize %d totalSize %d", listSize, headSize, totalSize);*/
   slog->shmid = shmget(SP_SHM_ID, SLOG_SHM_SIZE, (IPC_CREAT | 0666) );
   if (slog->shmid < 0)
   {
	  slog->shmid = shmget(SP_SHM_ID, 0, 0 );
      if (slog->shmid < 0)
      {
         hardw("shmget failed for %d", slog->shmid);
         sprintf(sqlStmtInfo,"shmget failed for %d", slog->shmid);
         return 0;
      } else
         hardw("Shared memory was already created %d", slog->shmid);
   } 
   if ((slog->shmmem = shmat(slog->shmid, (const void *)0, 0 )) == (void *) -1)
   {
       hardw("shmat failed for %d", slog->shmid);
       sprintf(sqlStmtInfo,"shmat failed for %d errno %d %s", slog->shmid, errno, strerror(errno));
       return 0;	   
   }
   slog->header = (SPhead *) slog->shmmem;
   strlwr(slog->name, getspName(slog->handle.spName));
   if (slog->header->gLogging > 0)
   {
      buildHashTable(slog);
      /*hardw("HashTable from memory. SHMID=%d TotalSize=%d HandleSize %d gLogging %c gLogSplit %c gOffset %d", 
             slog->shmid, totalSize, BYTEHANDLESIZE, slog->header->gLogging, 
            slog->header->gLogSplit, slog->header->gOffset);*/
   } else
   {
      parseSPConfig(slog);
      /*hardw("Hashtable from file SHMID=%d TotalSize=%d HandleSize %d gLogging %c gLogSplit %c gOffset %d", 
             slog->shmid, totalSize, BYTEHANDLESIZE, slog->header->gLogging, 
            slog->header->gLogSplit, slog->header->gOffset);*/
   }
   hardw("At the end: attachSHM shmmem %p", slog->shmmem);
   return 1;
}

int detachSHM(SLOG *slog)
{
   int retcode = shmdt((void *)slog->shmmem);   
   if (retcode < 0)
   {
      hardw("shmdt failed code= %d %s %d %s", retcode, (slog->shmmem == NULL) ? "shmmem is null" : "shmmem is not null", errno, strerror(errno));
      sprintf(sqlStmtInfo,"shmdt failed for %d errno %d %s", retcode, errno, strerror(errno));
      return retcode;
   } else
      return 1;
}

int DirExists(char *dirName)
{
   struct stat sb;
   if (stat(dirName, &sb) == 0) 
   {
      if (sb.st_mode == S_IFDIR) 
      {
         return 1;
      }
   }
   return 0;
}

void hardw(char *msg, ...)
{
   va_list argp;
#ifdef GOFILE
   FILE *p;
   char name[500];
#endif   

#ifdef GOFILE   
   if (DEBUG) 
   {
      strcpy(name, getenv("HOME"));
      if (name == NULL)
      {
         strcpy(name, "/tmp/hard.txt");
      } else
      {
         strcat(name, "/sqllib/hard.txt");
      }
      p = fopen(name, "a");
      if (p == NULL)
      {
         p = fopen("/tmp/hard.txt", "a");
      }
      if (p != NULL)
      {
         fprintf(p, "%s[%d]", getTimeTS(), getpid());
         va_start(argp, msg);
         vfprintf(p, msg, argp);
         va_end(argp);
         fprintf(p, "\n");
         fflush( p);
         fclose(p);
      } 
   }
#else
   if (DEBUG) 
   {
     fprintf(stdout,"%s[%d]", getTimeTS(), getpid());
     va_start(argp, msg);
     vfprintf(stdout,msg, argp);
     va_end(argp);
     fprintf(stdout,"\n");
   }
#endif   
} 

int isHashEmpty(SLOG *slog)
{
   return (slog->header == NULL) ? 0 : 1;
}

unsigned hash(char *s)
{
    unsigned hashval;

    for (hashval = 0; *s != '\0'; s++)
        hashval = toupper(*s) + 31*hashval;

    return hashval % HASHSIZE;
}

PList *lookup(SLOG *slog, char *s)
{
    PList *np;

    if (s == NULL || *s == '\0')
    	return NULL;
    for (np = slog->hashtable[hash(s)]; np != NULL; np = np->next)
        if (strcmp(s, np->name) == 0)
            return np;
    return NULL;
}

int infoMode(SLOG *slog, PList *pList)
{
   if (pList == NULL)
	  return 0;
   if (slog->header->gLogging != 'Y')
      return 0;
   /*hardw("infoMode %s debugmode = %c", pList->name, pList->debug);*/
   return 1;
}

int debugMode(SLOG *slog, PList *pList)
{
   if (pList == NULL)
	  return 0;
   if (slog->header->gLogging != 'Y')
      return 0;
   /*hardw("debugmode %s debugmode = %c", pList->name, pList->debug);*/
   return (pList->debug == 'Y') ? 1 : 0;
}

void *removeNode(SLOG *slog, char *name)
{
   int i;
   PList *next, *prev;
   unsigned hashval;
   

   i = 0;
   hashval = hash(name);
   hardw("remove node %s from hashval=%d", name, hashval);
   for (next = slog->hashtable[hashval]; next != NULL; next = next->next)
   {
      hardw("Remove hash=%d Actual[%d] %s", hashval, i, next->name);
      if (strcmp(name, next->name) == 0)
      {
         if (i == 0)
         {
        	 slog->hashtable[hashval] = next->next;
         } else
         {
            prev->next = next->next;
         }
      }
      prev = next;
      ++i;
   }
}

void *shm_malloc(SLOG *slog, int sz)
{
   slog->header->gOffset += sz;
   /*hardw("shm_malloc offset %d size %d", slog->header->gOffset, sz);*/
   return (void *) (slog->shmmem + slog->header->gOffset - sz);  
}

PList *install(SLOG *slog, char *name, char debug)
{
   PList *np;
   unsigned hashval;

   hardw("installing name %s", name);
   if ((np = lookup(slog,name)) == NULL)
   {
       np = (PList *)shm_malloc(slog, sizeof(*np));

       if (np == NULL)
           return NULL;
       else
       {
          strcpy(np->name, name);
       }

       hashval = hash(name);
       np->next = slog->hashtable[hashval];
       slog->hashtable[hashval] = np;
       hardw("in install hashval %d", hashval);
   }
   np->debug = debug;
   return np;
}

void FreeHashTable(SLOG *slog)
{
   int i;
   
   slog->header->gOffset = sizeof(SPhead);
   for (i = 0; i < HASHSIZE; ++i)
   {
      slog->hashtable[i] = NULL;
   }
}

int AdvanceChar()
{
   if (c != EOF)
       c = (uint)fgetc(fin);
   return c;
}

int SkipWhite()
{
    while (isspace((uint) c))
        c = (uint)fgetc(fin);
    return c;
}

int NextProperty()
{
    do
    {
        while (c != '\n' && c != '\r' && c != EOF)
            c = (uint)fgetc(fin);

        if (c == '\r')
            c = (uint)fgetc(fin);

        if (c == '\n')
            c = (uint)fgetc(fin);
    }
    while (isspace(c));

    return c;
}

char ParseChar()
{
    char flag = 'N';
    SkipWhite();

    if (c == 't' || c == 'T' || c == 'y' || c == 'Y' || c == '1')
        flag = 'Y';
    else if (c == 'f' || c == 'F' || c == 'n' || c == 'N' || c == '0')
        flag = 'N';

    NextProperty();
    return flag;
}

void getLogConfigFileName(char *dbName, char *logConfigFileName)
{
   getHomeDir(logConfigFileName, LOGFILESIZE);	
   if (logConfigFileName[strlen(logConfigFileName)-1] != '/') 
   {
      strcat(logConfigFileName, "/");
   }
   strcat(logConfigFileName, "splog");
   if (!DirExists(logConfigFileName)) 
   {
      mkdir(logConfigFileName, S_IRWXU | S_IRWXG | S_IRWXO);
   }         
   strcat(logConfigFileName, "/");
   strcat(logConfigFileName, dbName);
   if (!DirExists(logConfigFileName)) 
   {
      mkdir(logConfigFileName, S_IRWXU | S_IRWXG | S_IRWXO);
   }        
   strcat(logConfigFileName, "/");
   strcat(logConfigFileName, LOG_CONFIG_NAME);
}

void parseSPConfig(SLOG *slog)
{
   int i;
   char name[NAMESIZE+DBSIZE];
   char logConfigFileName[LOGFILESIZE];
   char debug;
   PList *entry;

   slog->header->gLogging = 'Y';
   slog->header->gLogSplit = 'Y';
   slog->header->gOffset = sizeof(SPhead);
   getLogConfigFileName(slog->dbName, logConfigFileName);
   if ((fin = fopen(logConfigFileName, "r")) != NULL)
   {
      FreeHashTable(slog);
      c = 0;
      AdvanceChar();

      while (c != EOF)
      {
          while (c == '#')
              NextProperty();

          i = 0;

          while (c != '=' && c != EOF && i < NAMESIZE)
          {
              name[i++] = (char)tolower(c);
              AdvanceChar();
          }

          name[i] = '\0';

          if (c == '=')
          {
              AdvanceChar();
              debug = ParseChar();
              if (strcmp(name, "global_logging") == 0)
              {
            	  slog->header->gLogging = debug;
              } else if (strcmp(name, "log_split") == 0)
              {
            	  slog->header->gLogSplit = debug;
              }
              else
              {
                 install(slog, name, debug);
              }
          }
          else
              NextProperty();
      }
      fflush(fin);
      fclose(fin);
      printHashTable(slog);
   }
}

void writeSPConfig(SLOG *slog)
{
   int i;
   char logConfigFileName[LOGFILESIZE];
   FILE *fout;
   PList *next;

   getLogConfigFileName(slog->dbName, logConfigFileName);
   
   if ((fout = fopen(logConfigFileName, "w")) != NULL)
   {
      fprintf(fout,"# *************************************************************************\n");
      fprintf(fout,"# ** (C) COPYRIGHT International Business Machines Corp. 2000 - 2005\n");
      fprintf(fout,"# ** All Rights Reserved.\n");
      fprintf(fout,"# **\n");
      fprintf(fout,"# ** Vikram Khatri vikram@zinox.com\n");
      fprintf(fout,"# ** Use it at your own risk. No warranties implied and no support available\n");
      fprintf(fout,"# *************************************************************************\n");
      fprintf(fout,"global_logging=%c\n", slog->header->gLogging);
      fprintf(fout,"log_split=%c\n", slog->header->gLogSplit);
      for (i = 0; i < HASHSIZE; ++i)
      {
          next = slog->hashtable[i];
  
          while(next)
          {
              fprintf(fout,"%s=%c\n", next->name, next->debug);
              next = next->next;
          }
      }
      fclose(fout);
   }
}

int filecopy(char *from, char *to)
{
   int nbytes;
   int status = -1;
   int fd1 = open(from,O_RDONLY);
   int fd2 = open(to,O_RDONLY | O_WRONLY | O_CREAT,
                  S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
   static char buffer[BUFSIZ];

   /*hardw("filecopy %d %d", fd1, fd2);*/
   if (fd1 >= 0 && fd2 >= 0)
   {
      status = 0;
      while ((nbytes = read(fd1,buffer,BUFSIZ)) > 0)
      if (write(fd2,buffer,nbytes) != nbytes)
      {
         status = -1;
         break;
      }

      if (nbytes == -1)
         status = -1;
   }

   if (fd1 >= 0)
      close(fd1);
   if (fd2 >= 0)
      close(fd2);
   return status;
}

int archiveLogFile(SLOG *slog, char *logFileName)
{
   char newLogFileName[LOGFILESIZE];
   DIR *dirp;
   struct dirent *entry;
   struct stat finfo;
   int count;
   
   struct direct **files;
   struct stat filestat;
   
   if (stat(logFileName, &filestat) == 0)
   {
      if (filestat.st_size > FILE_SIZE)
      {
    	 getHomeDir(newLogFileName, LOGFILESIZE);
         if (newLogFileName[strlen(newLogFileName)-1] != '/') 
         {
            strcat(newLogFileName, "/");
         }
         strcat(newLogFileName, "splog");
         if (!DirExists(newLogFileName)) 
         {
            mkdir(newLogFileName, S_IRWXU | S_IRWXG | S_IRWXO);
         }        
         strcat(newLogFileName, "/");
         strcat(newLogFileName, slog->dbName);
         if (!DirExists(newLogFileName)) 
         {
            mkdir(newLogFileName, S_IRWXU | S_IRWXG | S_IRWXO);
         }        
         strcat(newLogFileName, "/");
         strcat(newLogFileName, "archive");
         if (!DirExists(newLogFileName)) 
         {
            mkdir(newLogFileName, S_IRWXU | S_IRWXG | S_IRWXO);
         }
         count = 0;
         dirp = opendir(newLogFileName);
         while ((entry = readdir(dirp)) != NULL)
         {
            count++;
         }
         closedir(dirp);
         if (count >= 2) 
         {
            count -= 2;
         }
         sprintf(newLogFileName,"%s/%s%04d%s", newLogFileName, (slog->header->gLogSplit == 'Y') ? slog->name : "sp",(count+1),".log");
   	     hardw("in archiveLogFile before filecopy newLogFileName=%s", newLogFileName);
         if (filecopy (logFileName,newLogFileName) == 0)
         {
            remove(logFileName);
            return 0;
         }
      }
   }
   return 1;
}

void getLogFileName(SLOG *slog, char *logFileName)
{
   int result;
   char *pdest;

   getHomeDir(logFileName, LOGFILESIZE);
   if (logFileName[strlen(logFileName)-1] != '/') 
   {
      strcat(logFileName, "/");
   }
   strcat(logFileName, "splog");
   if (!DirExists(logFileName)) 
   {
      mkdir(logFileName, S_IRWXU | S_IRWXG | S_IRWXO);
   }        
   strcat(logFileName, "/");
   strcat(logFileName, slog->dbName);
   if (!DirExists(logFileName)) 
   {
      mkdir(logFileName, S_IRWXU | S_IRWXG | S_IRWXO);
   }        
   sprintf(logFileName,"%s/%s%s", logFileName, 
      (slog->header->gLogSplit == 'Y') ? slog->name : "sp", ".log"); 
   /*hardw("getLogFileName  %s", logFileName);*/
}

FILE *openLogFile(SLOG *slog, PList *p)
{
    FILE *fp;
    char logFileName[LOGFILESIZE];

    getLogFileName(slog, logFileName);
    archiveLogFile(slog, logFileName); 
	if ((fp = fopen(logFileName, "a")) == NULL) 
	{
	    sprintf(sqlStmtInfo, "File open error %d %s %s", errno, strerror(errno), strrchr(logFileName,'/')+1);
	    return NULL;
	}    		
    return fp;
}

/* CALL DB2.UPDATE_SP_CFG('PARAM_NAME','Y') */
int updateSPConfig(SLOG *slog, char *name, char *mode)
{
   char sname[NAMESIZE+DBSIZE], gLogging[NAMESIZE+DBSIZE], gLogSplit[NAMESIZE+DBSIZE];
   PList *entry;
   char flag = 'N';
   uint c = mode[0];

   strlwr(sname, name);
   sprintf(sname, "%s_%s", slog->dbName, name);
   sprintf(gLogging, "%s_%s", slog->dbName, "global_logging");
   sprintf(gLogSplit, "%s_%s", slog->dbName, "log_split");
   if (c == 't' || c == 'T' || c == 'y' || c == 'Y' || c == '1')
       flag = 'Y';
   else if (c == 'f' || c == 'F' || c == 'n' || c == 'N' || c == '0')
       flag = 'N';
   
   if (attachSHM(slog))
   {
      if (c == 'r' || c == 'R')
      {
         removeNode(slog,sname);
      }
      else
      {
         if (strcmp(sname, gLogging) == 0)
        	 slog->header->gLogging = flag; 
         else if(strcmp(sname, gLogSplit) == 0)
         {
        	 slog->header->gLogSplit = flag; 
         }
         else
            install(slog, sname, flag);
      }
      writeSPConfig(slog);
      if (c == 'r' || c == 'R')
      {
         /* Force rebuild of hashtable from file */
         parseSPConfig(slog);
      }
      hardw("updateSPConfig SHMID=%d gLogging %c gLogSplit %c gOffset %d", 
             slog->shmid, slog->header->gLogging, slog->header->gLogSplit, slog->header->gOffset);
      if (detachSHM(slog) < 0)
    	  return -1;
   } else
	   return -1;
   return 1;
}

char *getTimeTS()
{
   static char buf[128];
   char buffer[30];
   struct timeval tv;
   struct timeb localTime;

   time_t curtime;
   ftime(&localTime);
   tv.tv_sec = localTime.time + localTime.timezone;
   tv.tv_usec = localTime.millitm;
   curtime=tv.tv_sec;

   strftime(buffer,30,"%m-%d-%Y %H:%M:%S.",localtime(&curtime));
   sprintf(buf,"%s%03ld",buffer,tv.tv_usec);
   return buf;
}

/* CALL DB2.LOGGER(handle, 'Message') */
int logger(SLOG *slog, char *msg, ... )
{
   struct timeb prevTime;
   va_list argp;
   FILE *fp;
   PList *p;

   if (&slog->handle == NULL) return -1;
   if (attachSHM(slog))
   {
	  p = lookup(slog,slog->handle.spName);
      if (debugMode(slog,p)) 
      {
         if ((fp = openLogFile(slog,p)) == NULL)
         	return -1;
         fprintf(fp, "%s[%.3f]:[%s:%s] ", getTimeTS(), getElapsed(&slog->handle.prevTime), slog->name, slog->appID);
	     ftime(&prevTime);
	     memcpy(&slog->handle.prevTime, &prevTime, sizeof(prevTime));         	 
         va_start(argp, msg);
         vfprintf(fp, msg, argp);
         va_end(argp);
         fprintf(fp, "\n");
         fclose(fp);
      }
      if (detachSHM(slog) < 0)
    	  return -1;
   } else
	   return -1;
   return 1;
}

/* CALL DB2.LOGINFO(handle, 'Message') */
int loginfo(SLOG *slog, char *msg, ... )
{
   struct timeb prevTime;
   va_list argp;
   FILE *fp;
   PList *p;

   if (&slog->handle == NULL) return -1;
   if (attachSHM(slog)) 
   {
	  p = lookup(slog, slog->handle.spName);
      if (infoMode(slog,p)) 
      {
         if ((fp = openLogFile(slog,p)) == NULL)
        	return -1;
         /*if (getTimeTS() == NULL) hardw("getTimeTS() is null");
         if (&slog->handle.prevTime == NULL)  hardw("slog->handle.prevTime is null");
         if (slog->name == NULL) hardw("slog->name is null");
         if (slog->appID == NULL) hardw("slog->appID is null");
         if (msg == NULL) hardw("msg is NULL");
         if (fp == NULL) hardw("fp is null");
         else hardw("fp value = %p", fp);*/
         fprintf(fp, "%s[%.3f]:[%s:%s]", getTimeTS(), getElapsed(&slog->handle.prevTime), slog->name, slog->appID);
         ftime(&prevTime);
         memcpy(&slog->handle.prevTime, &prevTime, sizeof(prevTime));  
         /*hardw("startTime %d prevTime %d", slog->handle.startTime.time, slog->handle.prevTime.time);*/
         va_start(argp, msg);
         vfprintf(fp, msg, argp);
         va_end(argp);
         fprintf(fp, "\n");
         fclose(fp);
      }
      if (detachSHM(slog) < 0)
    	  return -1;
   } else
	   return -1;
   return 1;
}

/* CALL DB2.OPEN_LOG('SP_NAME', handle) */
int openLog(SLOG *slog, char *name)
{ 
   int rc;	
   char sname[NAMESIZE+DBSIZE];
   struct timeb startTime;
   PList *p;
   
   sprintf(sname, "%s_%s", slog->dbName, name);
   strlwr(sname, sname);   
   strcpy(slog->handle.spName, sname);
   if (attachSHM(slog))
   {
	     p = lookup(slog,sname);
	     if (p != NULL)
	     {
	    	 ftime(&startTime);
	    	 memcpy(&slog->handle.startTime, &startTime, sizeof(startTime)); 
	    	 memcpy(&slog->handle.prevTime, &startTime, sizeof(startTime)); 
	         if (detachSHM(slog) < 0)
	        	 return -1;
	         rc = loginfo(slog, "Begin Stored Procedure *** [%d]", slog->shmid);
	         /*hardw("openlog rc %d", rc);*/
	         return rc;
	     } else
	     {
	         if (detachSHM(slog) < 0) 
	         {
	        	hardw("From openlog:shmmem value %s", (slog->shmmem == NULL) ? "shmmem is null" : "shmmem is not null");
	        	return -1;
	         }
	     }
    }
    else
    {
   	   hardw("From openlog:shmmem value %s", (slog->shmmem == NULL) ? "shmmem is null" : "shmmem is not null");
       return -1;
    }
    return 1;
}

/* CALL DB2.CLOSE_LOG(handle) */
int closeLog(SLOG *slog)
{
   int rc;
   rc = loginfo(slog, "End   Stored Procedure *** Elapsed %.3f", getElapsed(&slog->handle.startTime));
   return rc;
}

SQL_API_RC SQL_API_FN UpdateSPConfig
(
   char inSPName[NAMESIZE],
   char inMode[2],
   sqlint16 *inSPNameNullInd,
   sqlint16 *inModeNullInd,
   SQLUDF_TRAIL_ARGS,
   SQLUDF_DBINFO *dbInfo
)
{
  SLOG slog;
  if(*inSPNameNullInd < 0 || inSPName[0] == 0) return -1;
  if(*inModeNullInd < 0 || inMode[0] == 0) return -1;
  initSlog(&slog, dbInfo);
  hardw("in UpdateSPConfig dbName %s %s", slog.dbName, sqludf_fspecname);
  if (updateSPConfig(&slog, inSPName, inMode) == -1)
  {
	 strcpy(sqludf_sqlstate, "38776");
	 sprintf(sqludf_msgtext, "%d:%s", -8000,sqlStmtInfo);
  }
  return 0;
}

SQL_API_RC SQL_API_FN OpenLog
(
   char inSPName[NAMESIZE],
   char outHandle[BYTEHANDLESIZE],
   sqlint16 *inSPNameNullInd,
   sqlint16 *outHandleNullInd,
   SQLUDF_TRAIL_ARGS,
   SQLUDF_DBINFO *dbInfo
)
{
  SLOG slog;

  hardw("in OpenLog SP");
  if(*inSPNameNullInd < 0 || inSPName == NULL) return -1;
  initSlog(&slog, dbInfo);
  memset((char *) &slog.handle, 0, BYTEHANDLESIZE);
  if (openLog(&slog, inSPName) == -1)
  {
	 strcpy(sqludf_sqlstate, "38777");
	 sprintf(sqludf_msgtext, "%d:%s", -8000,sqlStmtInfo);
	 return 0;	  
  }
  memcpy(outHandle, (char *) &slog.handle, BYTEHANDLESIZE);
  *outHandleNullInd = (outHandle == NULL) ? -1 : 0;
  return 0;  
}

SQL_API_RC SQL_API_FN CloseLog
(
   char inHandle[BYTEHANDLESIZE],
   sqlint16 *inHandleNullInd,
   SQLUDF_TRAIL_ARGS,
   SQLUDF_DBINFO *dbInfo
)
{
  SLOG slog;
  hardw("in CloseLog SP");
  if(*inHandleNullInd < 0 || inHandle == NULL) return -1;
  initSlog(&slog, dbInfo);
  memcpy(&slog.handle, (Handle *)inHandle, BYTEHANDLESIZE);
  if (closeLog(&slog) == -1)
  {
	strcpy(sqludf_sqlstate, "38778");
	sprintf(sqludf_msgtext, "%d:%s", -8000,sqlStmtInfo);
  }
  return 0;
}

SQL_API_RC SQL_API_FN Logger
(
   char inoutHandle[BYTEHANDLESIZE],
   char inMessage[4000],
   sqlint16 *inoutHandleNullInd,
   sqlint16 *inMessageNullInd,
   SQLUDF_TRAIL_ARGS,
   SQLUDF_DBINFO *dbInfo
)
{
  SLOG slog;
  hardw("in Logger SP %s ", inMessage);
  if(*inoutHandleNullInd < 0 || inoutHandle == NULL) 
     return -1;
  if(*inMessageNullInd < 0 || *inMessageNullInd == -1) 
     return -1;
  initSlog(&slog, dbInfo);
  memcpy(&slog.handle, (Handle *)inoutHandle, BYTEHANDLESIZE);
  if (logger(&slog, "%s", inMessage) == -1)
  {
	strcpy(sqludf_sqlstate, "38779");
	sprintf(sqludf_msgtext, "%d:%s", -8000,sqlStmtInfo);
  }
  memcpy(inoutHandle, (char *) &slog.handle, BYTEHANDLESIZE);
  return 0;
}

SQL_API_RC SQL_API_FN Loginfo
(
   char inoutHandle[BYTEHANDLESIZE],
   char inMessage[4000],
   sqlint16 *inoutHandleNullInd,
   sqlint16 *inMessageNullInd,
   SQLUDF_TRAIL_ARGS,
   SQLUDF_DBINFO *dbInfo
)
{
  SLOG slog;	
  hardw("in Loginfo SP %d %s", BYTEHANDLESIZE, inMessage);
  if(*inoutHandleNullInd < 0 || inoutHandle == NULL) return -1;
  if(*inMessageNullInd < 0 || *inMessageNullInd == -1) return -1;
  initSlog(&slog, dbInfo);
  memcpy(&slog.handle, (Handle *)inoutHandle, BYTEHANDLESIZE);
  if (loginfo(&slog, "%s", inMessage) == -1)
  {
	strcpy(sqludf_sqlstate, "38780");
	sprintf(sqludf_msgtext, "%d:%s", -8000,sqlStmtInfo);
  }
  memcpy(inoutHandle, (char *) &slog.handle, BYTEHANDLESIZE);
  return 0;
}

int main(int argc, char *argv[])
{
   int i;
   char handle[BYTEHANDLESIZE];
   char msgloginfo[100], msglogger[100];
   char appID[APPIDSIZE];
   sqlint16 indValue = 0;
   SQLUDF_DBINFO dbInfo;

   strcpy((char *) dbInfo.dbname, "falcon");
   dbInfo.dbnamelen = 6;
   strcpy(appID, "G91140FB.BEF7.");
   sprintf(appID, "%s%ld", appID, getpid());
   dbInfo.appl_id = strdup(appID);
   hardw("Start %s", dbInfo.dbname);

   /*************** Test case 1 **********
   OpenLog("testsp", handle, &indValue,&indValue,"000000","LOG_SPLIT","OPENLOG_TRUSTED","",&dbInfo);
   for (i = 0; i < 3000; ++i)
   {
	 sprintf(msgloginfo,"This is a loginfo %d message", (i+1));
	 sprintf(msglogger,"This is a logger %d message", (i+1));
     Loginfo(handle, msgloginfo,&indValue,&indValue,"000000","LOG_SPLIT","LOGINFO_TRUSTED","",&dbInfo);
     Logger(handle, msglogger,&indValue,&indValue,"000000","LOG_SPLIT","LOGGER_TRUSTED","",&dbInfo);
   }
   CloseLog(handle,&indValue,"000000","LOG_SPLIT","CLOSELOG_TRUSTED","",&dbInfo);
   ***************************************/
   
   /*************** Test case 2 **********/
   UpdateSPConfig(argv[1],argv[2],&indValue,&indValue,"000000","LOG_SPLIT","UPDSPCFG_TRUSTED","",&dbInfo);
   /***************************************/
   return 0;
}

